require 'spec_helper'
require 'emcee'
require 'emcee/action'
require 'emcee/runner'
require 'emcee/agent'
require 'emcee/result'

describe Emcee::Action do
  before do
    @runner = Emcee::Runner.new
    @agent = Emcee::Agent.new(@runner, :agent => 'someagent', :version => '1.0')
    @interface = {
      :output => { :output   => { :display_as => "Output", :description => "Output from the Nagios plugin" },
                   :exitcode => { :display_as => "Exit Code", :description => "Exit Code from the Nagios plugin" },
                   :perfdata => { :display_as => "Performance Data", :description => "Performance Data from the Nagios plugin" }
      },
      :action      => "runcommand",
      :description => "Run a NRPE command",
      :display => :failed,
      :input => {
        :command => {
          :type        => :string,
          :description => "NR PE command to run",
          :prompt      => "Command",
          :optional    => false,
          :validation  => "^[a-zA-Z0-9_-]+$",
          :maxlength   => 50
        }
      }
    }
  end

  describe 'when initializing' do
    it 'accepts an agent and a DDL action interface hash' do
      lambda { Emcee::Action.new(@agent, @interface) }.should_not raise_error(ArgumentError)
    end

    it 'requires an agent and a DDL action interface hash' do
      lambda { Emcee::Action.new(@agent) }.should raise_error(ArgumentError)
    end

    it 'makes the agent available' do
      Emcee::Action.new(@agent, @interface).agent.should == @agent
    end

    it 'makes the interface hash available' do
      Emcee::Action.new(@agent, @interface).interface.should == @interface
    end

    it 'makes the agent name available' do
      Emcee::Action.new(@agent, @interface).name.should == @interface[:action]
    end

    it 'makes the agent description available' do
      Emcee::Action.new(@agent, @interface).description.should == @interface[:description]
    end

    it 'makes the agent display available' do
      Emcee::Action.new(@agent, @interface).display.should == @interface[:display]
    end

    it 'makes the agent input description available' do
      Emcee::Action.new(@agent, @interface).input.should == @interface[:input]
    end

    it 'makes the agent output description available' do
      Emcee::Action.new(@agent, @interface).output.should == @interface[:output]
    end
  end

  describe 'determining if the action requires arguments' do
    it 'returns false if the input specification is empty' do
      @interface[:input] = {}
      Emcee::Action.new(@agent, @interface).requires_arguments?.should be_false
    end

    it 'returns true if the input specification has entries' do
      Emcee::Action.new(@agent, @interface).requires_arguments?.should be_true
    end
  end

  describe 'running an action' do
    before do
      @wrapped_result = 'wrapped result'
      @result = 'action result'
    end

    it 'fails if the action is unknown' do
      @action = Emcee::Action.new(@agent, @interface.merge(:action => 'completelyunknown'))
      @runner.stubs(:rpcclient).raises(Exception)
      lambda { @action.run }.should raise_error
    end

    it 'returns Emcee::Results which wrap the RPC results for the agent action' do
      Emcee::Result.stubs(:new).with(@result).returns(@wrapped_result)
      @runner.stubs(:run).returns([ @result ])

      @action = Emcee::Action.new(@agent, @interface)
      @action.run.should == [ @wrapped_result ]
    end
  end
end

