require 'spec_helper'
require 'emcee'
require 'emcee/null_cache'

class Emcee::CacheBase
  def log(message)
  end
end

describe Emcee::NullCache do
  before do
    @key = { 'foo' => { 'bar' => 'baz' }, 'xyzzy' => [ 1, 2, 3, [ 'a', 'b', 'c' ] ] }
  end

  describe 'when initializing' do
    it 'accepts a key' do
      lambda { Emcee::NullCache.new(@key) }.should_not raise_error(ArgumentError)
    end

    it 'requires a key' do
      lambda { Emcee::NullCache.new }.should raise_error(ArgumentError)
    end

    it 'makes the key available after initialization' do
      Emcee::NullCache.new(@key).key.should == @key
    end
  end

  describe 'when accessing data' do
    before do
      @cache = Emcee::NullCache.new(@key)
    end

    describe 'and data has not been seen before' do
      describe 'when a value is given' do
        it 'returns the value given' do
          @cache.data('foo').should == 'foo'
        end
      end

      describe 'when no value is given but a block is given' do
        it 'runs the data block' do
          lambda { @cache.data do raise "flunk" end }.should raise_error(RuntimeError)
        end

        it 'returns the data' do
          @cache.data do [ 'x', 'y' ] end.should == [ 'x', 'y' ]
        end
      end
    end

    describe 'and data has been seen before' do
      before do
        @cache.data do [ 'x', 'y' ] end
      end

      describe 'when a value is given' do
        it 'returns the value given' do
          @cache.data('foo').should == 'foo'
        end
      end

      describe 'when no value is given but a block is given' do
        it 'runs the data block' do
          lambda { @cache.data do raise "flunk" end }.should raise_error(RuntimeError)
        end

        it 'returns the data' do
          @cache.data do [ 'x', 'y' ] end.should == [ 'x', 'y' ]
        end
      end
    end
  end

  describe 'when manually expiring the cache' do
    before do
      @cache = Emcee::NullCache.new(@key)
    end

    it 'does not fail' do
      lambda { @cache.expire! }.should_not raise_error
    end
  end

  describe 'when checking if a value is cached' do
    before do
      @cache = Emcee::NullCache.new(@key)
    end

    it 'is false when the value has not been seen before' do
      @cache.cached?.should be_false
    end

    it 'is false when the value has been seen before' do
      @cache.data('foo')
      @cache.cached?.should be_false
    end
  end
end
